/* This file is part of Netwib.
*/

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_init_pathname(netwib_constbuf *pathname,
                                          struct stat *pst)
{
  netwib_string str;

  netwib__buf_ref_string(pathname, str, bufstorage,
                         netwib_priv_stat_init_pathname(&bufstorage, pst));

  return(netwib_priv_stat_init_pathname2(str, pst));
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_init_pathname2(netwib_conststring pathname,
                                           struct stat *pst)
{
  netwib_buf buf;
  int reti;

  if (pst == NULL) {
    return(NETWIB_ERR_PANULLPTR);
  }

#if defined NETWIBDEF_SYSNAME_Unix
  reti = lstat(pathname, pst);
#elif defined NETWIBDEF_SYSNAME_Windows
  reti = _stat(pathname, (struct _stat *)pst);
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif
  if (reti == -1) {
    if (errno == ENOENT) {
      netwib_er(netwib_priv_errmsg_reinit());
      netwib_er(netwib_buf_init_ext_text("file not found : ", &buf));
      netwib_er(netwib_priv_errmsg_append_buf(&buf));
      netwib_er(netwib_buf_init_ext_text(pathname, &buf));
      netwib_er(netwib_priv_errmsg_append_buf(&buf));
      return(NETWIB_ERR_OKNOTFOUND);
    }
    return(NETWIB_ERR_FULSTAT);
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_init_fd(int fd,
                                    struct stat *pst)
{
  int reti;

  if (pst == NULL) {
    return(NETWIB_ERR_PANULLPTR);
  }

#if defined NETWIBDEF_SYSNAME_Unix
  reti = fstat(fd, pst);
#elif defined NETWIBDEF_SYSNAME_Windows
  reti = _fstat(fd, (struct _stat *)pst);
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif
  if (reti == -1) {
    return(NETWIB_ERR_FUFSTAT);
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_decode_size(struct stat st,
                                        netwib_uint32 *psize)
{

#if defined NETWIBDEF_SYSNAME_Unix
  if (!S_ISREG(st.st_mode)) {
    return(NETWIB_ERR_SPFILENOTREG);
  }
#elif defined NETWIBDEF_SYSNAME_Windows
  if (! (st.st_mode & _S_IFREG) ) {
    return(NETWIB_ERR_SPFILENOTREG);
  }
#else
 #error "Unknown value for NETWIBDEF_SYSNAME"
#endif

#if NETWIBDEF_ARCH_BITS == 64
  if (st.st_size > 0xFFFFFFFFu) {
    return(NETWIB_ERR_SPFILE4G);
  }
#endif

  /* set the correct value */
  if (psize != NULL) {
    *psize = (netwib_uint32) st.st_size;
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_decode_type(struct stat st,
                                        netwib_pathnametype *ptype)
{
  if (ptype == NULL) {
    return(NETWIB_ERR_OK);
  }

#if defined NETWIBDEF_SYSNAME_Unix
  if (S_ISREG(st.st_mode))
    *ptype = NETWIB_PATHNAMETYPE_REG;
  else if (S_ISDIR(st.st_mode))
    *ptype = NETWIB_PATHNAMETYPE_DIR;
  else if (S_ISLNK(st.st_mode))
    *ptype = NETWIB_PATHNAMETYPE_LINK;
  else if (S_ISSOCK(st.st_mode))
    *ptype = NETWIB_PATHNAMETYPE_SOCK;
  else if (S_ISBLK(st.st_mode))
    *ptype = NETWIB_PATHNAMETYPE_BLOCK;
  else if (S_ISCHR(st.st_mode))
    *ptype = NETWIB_PATHNAMETYPE_CHAR;
  else if (S_ISFIFO(st.st_mode))
    *ptype = NETWIB_PATHNAMETYPE_FIFO;
  else
    *ptype = NETWIB_PATHNAMETYPE_UNKNOWN;
#elif defined NETWIBDEF_SYSNAME_Windows
  if (st.st_mode & _S_IFREG)
    *ptype = NETWIB_PATHNAMETYPE_REG;
  else if (st.st_mode & _S_IFDIR)
    *ptype = NETWIB_PATHNAMETYPE_DIR;
  else
    *ptype = NETWIB_PATHNAMETYPE_UNKNOWN;
#else
 #error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_forbid_change(struct stat st1,
                                          struct stat st2)
{
  if (st1.st_ino != st2.st_ino || st1.st_nlink != st2.st_nlink ||
#if defined NETWIBDEF_SYSNAME_Unix
      st1.st_dev != st2.st_dev || st1.st_mode != st2.st_mode ||
#elif defined NETWIBDEF_SYSNAME_Windows
      /* because fstat() returns a st_dev of zero */
      /* because mode of a .exe is 777 before and 666 when reading */
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif
      st1.st_uid != st2.st_uid || st1.st_gid != st2.st_gid ||
      st1.st_size != st2.st_size) {
    return(NETWIB_ERR_SPFILECHANGED);
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_forbid_link(struct stat st)
{
#if defined NETWIBDEF_SYSNAME_Unix
  if (S_ISLNK(st.st_mode)) {
    return(NETWIB_ERR_SPFILESYMLINK);
  }
#elif defined NETWIBDEF_SYSNAME_Windows
  /* links under Windows are only files named "x.lnk" */
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  return(NETWIB_ERR_OK);
  /* just for the compiler warning */
  st.st_mode = st.st_mode;
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_forbid_notreg(struct stat st)
{
#if defined NETWIBDEF_SYSNAME_Unix
  if (!S_ISREG(st.st_mode))
#elif defined NETWIBDEF_SYSNAME_Windows
  if (!(st.st_mode & _S_IFREG))
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif
  {
    return(NETWIB_ERR_SPFILENOTREG);
  }
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_stat_display(struct stat st)
{
  netwib_er(netwib_fmt_display("dev=%{uint32} ", st.st_dev));
  netwib_er(netwib_fmt_display("ino=%{uint32} ", st.st_ino));
  netwib_er(netwib_fmt_display("mode=%{uint32} ", st.st_mode));
  netwib_er(netwib_fmt_display("nlink=%{uint32} ", st.st_nlink));
  netwib_er(netwib_fmt_display("uid=%{uint32} ", st.st_uid));
  netwib_er(netwib_fmt_display("gid=%{uint32} ", st.st_gid));
  netwib_er(netwib_fmt_display("size=%{uint32}\n", st.st_size));

  return(NETWIB_ERR_OK);
}

